#ifdef PHOTOLINE
#include "datatype.h"
#endif

#include "../config.h"
#include "../lensfun.h"
#ifndef __G_LIB_H__
#include "glib.h"
#endif
#include <ctype.h>
#include <wctype.h>
#include <regex.h>

static gchar _homeDir[2048];
static GLibDelegate	*_delegate = NULL;

LF_EXPORT void g_set_delegate(GLibDelegate *del)
{
	_delegate = del;
}

int g_parse_lens_name(const char *model, float &minF, float &maxF, float &minA)
{
	return(_delegate->parseLensName(model, &minF, &maxF, &minA));
}

#ifndef PLATFORM_WINDOWS
char *_strdup(const char *str)
{
	return(g_strdup(str));
}

int _write(int _FileHandle, void const *_Buf, unsigned int _MaxCharCount)
{
	return(0);
}

int _close(int _FileHandle)
{
	return(0);
}
#endif

static gchar *g_strdup_with_add(const gchar *str, size_t add)
{
	gchar		*ret = NULL;

	if (str)
	{
		size_t	length = strlen(str) + add;

		ret = (gchar *)g_malloc(length + 1);
		if (ret)
			strcpy(ret, str);
	}
	return(ret);
}

LF_EXPORT gchar *g_strdup(const gchar *str)
{
	return(g_strdup_with_add(str, 0));
}

void *g_malloc(size_t size)
{
	// ist aso dokumentiert: size == 0 liefert NULL
	if (size)
		return(malloc(size));
	return(NULL);
}

void g_free(gpointer mem)
{
	free(mem);
}

void *g_realloc(void *p, size_t len)
{
	return(realloc(p, len));
}

char *g_markup_vprintf_escaped(const char *format, va_list args)
{
	return(NULL);
}

void g_string_append(GString *output, const char *s)
{
}

gunichar g_utf8_get_char(const char *c)
{
	unsigned int	uniChar;
	
	_delegate->readUTF8Char(c, &uniChar);
	return((gunichar)uniChar);
}

const char *g_utf8_next_char(const char *c)
{
	unsigned int	uniChar;
	
	c = _delegate->readUTF8Char(c, &uniChar);
	return(c);
}

int g_unichar_isspace(gunichar c)
{
	return(isspace((int)c));
}

gunichar g_unichar_tolower(gunichar c)
{
	return((gunichar)towlower((int)c));
}

char *g_utf8_casefold(const char *c, int d)
{
	return(_delegate->casefoldUTF8(c, d));
}

gboolean g_str_has_prefix(const gchar *str, const gchar *prefix)
{
	return(TRUE);
}

GPtrArray *g_ptr_array_new()
{
	GPtrArray	*ret = new GPtrArray;

	ret->pdata = NULL;
	ret->len = 0;

	return(ret);
}

void g_ptr_array_add(GPtrArray *array, gpointer data)
{
	g_ptr_array_set_size(array, array->len + 1);
	array->pdata[array->len - 1] = data;
}

void g_ptr_array_set_size(GPtrArray *arr, guint length)
{
	gpointer	*newMem = (gpointer *)g_realloc(arr->pdata, (size_t)length * sizeof(gpointer *));
	guint		i;
	
	for (i = arr->len; i < length; i++)
		newMem[i] = NULL;
	
	arr->pdata = newMem;
	arr->len = length;
}

#if 0
gpointer g_ptr_array_index(GPtrArray *arr, guint index)
{
	gpointer	ret = NULL;
	
	if (arr->pdata)
		ret = arr->pdata[index];
	
	return(ret);
}
#endif

void g_ptr_array_remove_range(GPtrArray *arr, guint index, guint length)
{
	size_t	copyCount = (size_t)(arr->len - (index + length));
	memmove(arr->pdata + index, arr->pdata + index + length, copyCount * sizeof(gpointer *));
	arr->len -= length;
}

gpointer g_ptr_array_remove_index_fast(GPtrArray *arr, guint index)
{
	gpointer	ret = g_ptr_array_index(arr, index);
	
	if (arr->pdata)
	{
		arr->pdata[index] = arr->pdata[arr->len - 1];
		arr->len--;
	}
	return(ret);
}

gpointer*  g_ptr_array_free(GPtrArray *array, gboolean free_seg)
{
	gpointer	*ret = NULL;

	if (!free_seg)
		ret = array->pdata;
	else
		g_free(array->pdata);
	
	delete array;

	return(ret);
}

GString* g_string_sized_new(gsize dfl_size)
{
	return(NULL);
}

gchar *g_string_free(GString *string, gboolean free_segment)
{
	return(NULL);
}

GDir *g_dir_open(const char *dir, int, int *)
{
	return(_delegate->searchStart(dir));
}

const char *g_dir_read_name(GDir *dir)
{
	const char	*ret = _delegate->searchNext(dir);
	
	return(ret);
}

void g_dir_close(GDir *dir)
{
	_delegate->searchEnd(dir);
}

char *g_build_filename(const char *base, ...)
{
	char			*ret = g_strdup(base);
	va_list		ap;
	const char	*para;

	va_start(ap, base);
	while ((para = va_arg(ap, const char *)) != NULL)
	{
		size_t	len = strlen(para) + 1;
		char		*local;
		
		local = g_strdup_with_add(ret, len);
		if (local)
		{
			strcat(local, "/");
			strcat(local, para);
			g_free(ret);
			ret = local;
		}
	}

	va_end(ap);

	return(ret);
}

LF_EXPORT void g_set_user_data_dir(const gchar *path)
{
	if (path)
	{
		size_t	len;

		len = strlen(path);
		if (len < (sizeof(_homeDir) - 1))
			strcpy(_homeDir, path);
	}
}

const gchar *g_get_user_data_dir(void)
{
	return(_homeDir);
}

static size_t g_file_get_length(const gchar *filename)
{
	size_t	ret = 0;
	FILE		*file = fopen(filename, "rb");
	
	if (file)
	{
		fseek(file, 0, SEEK_END);
		
		ret = (size_t)ftell(file);
		
		fclose(file);
	}
	return(ret);
}

static GError	_error;

gboolean g_file_get_contents(const gchar *filename, gchar **contents, gsize *lengthPtr, GError **error)
{
	gboolean	ret = false;
	size_t	length = g_file_get_length(filename);
	
	if (length > 0)
	{
		FILE	*file = fopen(filename, "rb");
		if (file)
		{
			*contents = (gchar *)g_malloc(length + 1);
			fread(*contents, length, 1, file);
			fclose(file);
			
			(*contents)[length] = 0;
			if (lengthPtr)
				*lengthPtr = length;
			
			ret = true;
		}
	}
	else
	{
		if (error)
		{
			*error = &_error;
			_error.code = G_FILE_ERROR_ACCES;
		}
	}
	return(ret);
}

int g_mkdir(const gchar *filename, int mode)
{
	return(0);
}

int g_open(const gchar *filename, int flags, int mode)
{
	return(0);
}

gboolean g_file_test(const gchar *filename, GFileTest test)
{
	return(_delegate->fileTest(filename, test) != 0);
}

int regcomp(regex_t *preg, const char *regex, int cflags)
{
	return(0);
}

int regexec(const regex_t *preg, const char *string, size_t nmatch, regmatch_t pmatch[], int eflags)
{
	return(1);
}

void regfree(regex_t *preg)
{
}

int g_static_mutex_get_mutex(int *m)
{
	return(0);
}

void g_mutex_lock(int v)
{
}

void g_static_mutex_unlock(int *v)
{
}

GPatternSpec *g_pattern_spec_new(const char *v)
{
	return(g_strdup(v));
}

int g_pattern_match(GPatternSpec *pspec, guint string_length, const gchar *string, const gchar *string_reversed)
{
	return(_delegate->matchWildCards(pspec, string));
}

void g_pattern_spec_free(GPatternSpec *pspec)
{
	g_free(pspec);
}

void g_set_error(GError **err, GQuark domain, gint code, const gchar *format, ...)
{
}

GQuark g_markup_error_quark(void)
{
	return(GQuark());
}

void g_markup_parse_context_get_position(GMarkupParseContext *context, gint *line_number, gint *char_number)
{
}

GMarkupParseContext *g_markup_parse_context_new(const GMarkupParser *parser, GMarkupParseFlags flags, gpointer user_data, GDestroyNotify user_data_dnotify)
{
	GMarkupParseContext	*ret = new GMarkupParseContext;
	
	ret->parser = parser;
	ret->userData = user_data;
	
	return(ret);
}

gboolean g_markup_parse_context_parse(GMarkupParseContext *context, const gchar *text, signed long long text_len, GError **error)
{
	gboolean	ret = _delegate->parseXML(context, text, (size_t)text_len) != 0;
	
	if (!ret && error)
		*error = &_error;
	return(ret);
}

const gchar *g_markup_parse_context_get_element(GMarkupParseContext *context)
{
	const gchar	*ret = _delegate->parseXMLGetElement(context);
	return(ret);
}

void  g_markup_parse_context_free(GMarkupParseContext *context)
{
	delete context;
}

